#include <gdk-pixbuf/gdk-pixbuf.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <errno.h>
#include "mandelbrotfunction.h"

int main(int argc, char** argv) {
    g_type_init();
    
    // parameters
    double min_re = -1.5;
    double max_re = 1.5;
    double min_im = -1.5;
    double max_im = 1.5;
    int max_iter = 500;
    int resolution = 100;
    
    
    // read command line arguments
    while (*++argv) {
        char* option = *argv;
        if (strcmp(option,"-h") == 0) {
            printf("Usage: \n");
            printf("  -l number: set real      minimum\n");
            printf("  -r number: set real      maximum\n");
            printf("  -b number: set imaginary minimum\n");
            printf("  -t number: set imaginary maximum\n");
            printf("  -p number: pixels per real/imaginary unit\n");
            printf("  -h       : print usage\n");
        }
        else { // handle parameters
            if (strcmp(option,"-p") == 0) {
                parse_int_par(&resolution, ++argv);
            }
            else if (strcmp(option,"-i") == 0) {
                parse_int_par(&max_iter, ++argv);
            }
            else if (strcmp(option,"-b") == 0) {
                parse_double_par(&min_im, ++argv);
            }
            else if (strcmp(option,"-t") == 0) {
                parse_double_par(&max_im, ++argv);
            }
            else if (strcmp(option,"-l") == 0) {
                parse_double_par(&min_re, ++argv);
            }
            else if (strcmp(option,"-r") == 0) {
                parse_double_par(&max_re, ++argv);
            }
            else {
                printf("Unknown option %s\n", option);
                exit(1);
            }
        }
    } 
    
    if (max_im <= min_im) {
        printf("Error: max_im <= min_im\n");
        exit(1);
    }
    if (max_re <= min_re) {
        printf("Error: max_re <= min_re\n");
        exit(1);
    }
    
    int width = (max_re - min_re) * resolution;
    int height = (max_im - min_im) * resolution;
    
    
    // mat contains number of iterations for each point
    int** mat;
    // initialize matrix
    mat = malloc(width * sizeof(int*));
    for (int i = 0; i < width; ++i) {
        mat[i] = malloc(height * sizeof(int));
    }
    // fill matrix
    for (int x_pos = 0; x_pos < width; ++x_pos) {
        for (int y_pos = 0; y_pos < height; ++y_pos) {
            // calculate coordinates for pixel
            double re = min_re + x_pos/ (double) resolution;
            double im = max_im - y_pos/ (double) resolution;
            
            // iterations
            mat[x_pos][y_pos] = get_num_iter(re, im, max_iter);
        }
    }
    
    // A row in the matrix contains iterations for a vertical line (column) of pixels:
    // Pixels in a column of the picture have the same x-coordinate 
    // and are therefore in the same row of the matrix.
    
    // create image buffer:
    GdkPixbuf* pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, width, height);
    
    int n_channels = gdk_pixbuf_get_n_channels (pixbuf); // == 3, because is_alpha(pixbuf) == FALSE
    int rowstride = gdk_pixbuf_get_rowstride (pixbuf);
    guchar* pixels = gdk_pixbuf_get_pixels (pixbuf);
    
    for (int y = 0; y < height; ++y) {
        for (int x = 0; x < width; ++x) {
            // get pixel p
            guchar* p = pixels + y * rowstride + x * n_channels;
            // set RGB
          
            if (mat[x][y] != 0) {
		p[0] = 0;  // red
                p[1] = pow(((double) mat[x][y]) / max_iter, 0.2) * 255; // green
		p[2] = 0; // blue
            }
            
        }
    }
    
    // save image
    gdk_pixbuf_save (pixbuf, "mandelbrot.jpg", "jpeg", NULL, "quality", "100", NULL);
}
